<?php

namespace App\Admin;

use App\Entity\User;
use Sonata\AdminBundle\Admin\AbstractAdmin;
use Sonata\AdminBundle\Datagrid\DatagridMapper;
use Sonata\AdminBundle\Datagrid\ListMapper;
use Sonata\AdminBundle\Form\FormMapper;
use Sonata\AdminBundle\Show\ShowMapper;
use Sonata\AdminBundle\Form\Type\ModelType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use App\Entity\Clientes;

class ClientesAdmin extends AbstractAdmin
{
    protected function configureDatagridFilters(DatagridMapper $filter): void
    {
        $filter
            ->add('provincia')
            ->add('activo')
            ;
    }

    protected function configureFormFields(FormMapper $formMapper): void
    {
        $formMapper
            ->add('nombre')
            ->add('email')
            ->add('direccion')
            ->add('telefono')
            ->add('codigoPostal', null, [
                'label' => 'Código Postal',
            ])
            ->add('provincia', ChoiceType::class, [
                'choices' => Clientes::getProvinciasLista(),
                'label' => 'Provincias',
            ])
            ->add('usuarios', ModelType::class, [
                'class' => 'App\Entity\User',
                'property' => 'username',
                'multiple' => true,
                'by_reference' => false,
                'btn_add' => false,
                'required' => false,
            ])
            ->add('activo')
        ;
    }

    protected function configureListFields(ListMapper $listMapper): void
    {
        $listMapper
            ->addIdentifier('id')
            ->add('nombre')
            ->add('email')
            ->add('direccion')
            ->add('telefono')
            ->add('codigoPostal', null, [
                'label' => 'Código Postal',
            ])
            ->add('provincia')
            ->add('activo', null, ['editable' => true])
            ->add('_action', 'actions', [
                'actions' => [
                    'show' => [],
                    'edit' => [],
                ],
                'label' => 'Acciones',
            ]);
    }

    protected function configureShowFields(ShowMapper $showMapper): void
    {
        $showMapper
            ->add('id')
            ->add('nombre')
            ->add('email')
            ->add('direccion')
            ->add('telefono')
            ->add('codigoPostal', null, [
                'label' => 'Código Postal',
            ])
            ->add('provincia')
            ->add('usuarios', null, [
                'associated_property' => 'username'
            ])
            ->add('activo');
    }
}