<?php

namespace App\Admin;

use Sonata\AdminBundle\Admin\AbstractAdmin;
use Sonata\AdminBundle\Route\RouteCollectionInterface;
use Sonata\AdminBundle\Datagrid\ListMapper;
use Sonata\AdminBundle\Form\FormMapper;
use Sonata\AdminBundle\Datagrid\DatagridMapper;
use Symfony\Component\Form\Extension\Core\Type\TextareaType;
use Symfony\Component\Form\Extension\Core\Type\TextType;
use Symfony\Component\Form\Extension\Core\Type\ChoiceType;
use FOS\CKEditorBundle\Form\Type\CKEditorType;
use Symfony\Component\Security\Core\Authorization\AuthorizationCheckerInterface;

class EmailsAdmin extends AbstractAdmin
{

    private $authorizationChecker;

    public function __construct(AuthorizationCheckerInterface $authorizationChecker)
    {
        $this->authorizationChecker = $authorizationChecker;
    }

    protected function configureActionButtons(array $buttonList, $action, $object = null): array
    {
        if (!$this->authorizationChecker->isGranted('ROLE_SUPER_ADMIN')) {
            unset($buttonList['create']);
        }

        return $buttonList;
    }

    protected function configureFormFields(FormMapper $formMapper): void
    {
        $subject = $this->getSubject();

        if ($subject && $subject->getId()) {
            $formMapper
                ->add('tipo', null, [
                    'disabled' => true,
                ]);
        } else {
            $formMapper
                ->add('tipo', ChoiceType::class, [
                    'choices' => [
                        'Pendiente' => 'pendiente',
                        'Validado' => 'validado',
                        'Preparando' => 'preparando',
                        'Enviando' => 'enviando',
                        'Enviado Cliente' => 'enviadoCliente',
                        'Enviado Fabricante' => 'enviadoFabricante',
                        'Cancelado' => 'cancelado',
                    ],
                ]);
        }

        $formMapper
            ->add('asunto')
            ->add('texto', CKEditorType::class, [
                'config' => [
                    'language' => 'es',
                ],
                'help' => '[identificador], [nombreCliente], [nombreFabricante], [fechaActualizacion], [horaActualizacion]',
            ])
        ;
    }

    protected function configureDatagridFilters(DatagridMapper $datagridMapper): void
    {
        $datagridMapper
            ->add('tipo')
        ;
    }

    protected function configureListFields(ListMapper $listMapper): void
    {
        $listMapper
            ->add('tipo', null, ['template' => 'admin/tipos_email.html.twig'])
            ->add('asunto')
            ->add('_action', 'actions', [
                'actions' => [
                    'edit' => [],
                ],
                'label' => 'Acciones',
            ])
        ;
    }
}