<?php

namespace App\Command;

use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Input\InputOption;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Style\SymfonyStyle;
use App\Repository\EventosRepository;
use App\Services\Mailer;

class RecordatoriosEventosCommand extends Command
{
    protected static $defaultName = 'app:enviar-recordatorio-evento';
    protected static $defaultDescription = 'Enviar recordatorios de los eventos del día siguiente.';

    public function __construct(EventosRepository $eventosRepository, Mailer $mailer)
    {
        $this->eventosRepository = $eventosRepository;
        $this->mailer = $mailer;

        parent::__construct();
    }

    protected function configure(): void
    {
        $this->setName('app:enviar-recordatorio-evento')
            ->setDescription('Enviar recordatorios de los eventos del día siguiente.')
        ;
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $io = new SymfonyStyle($input, $output);
        $start = new \DateTime('tomorrow');
        $start->setTime(0, 0, 0);
        $end = new \DateTime('tomorrow');
        $end->setTime(23, 59, 59);

        $eventos = $this->eventosRepository->findByDateRange($start, $end);

        if (count($eventos) === 0) {
            $io->success('No hay eventos para mañana.');
            return Command::SUCCESS;
        }

        try {
            $this->mailer->sendRecordatorio($eventos, $start);
        } catch (TransportExceptionInterface $e) {
            $io->error('El comando a fallado: ' . $e->getMessage());
            return false;
        }

        $io->success('El comando se ha ejecutado correctamente.');

        return Command::SUCCESS;
    }
}
