<?php

namespace App\Controller;

use Symfony\Bundle\FrameworkBundle\Controller\AbstractController;
use Symfony\Component\HttpFoundation\Response;
use Doctrine\Persistence\ManagerRegistry as PersistanceManagerRegistry;
use App\Repository\PedidosRepository;
use App\Entity\Ventas;


class DashboardController extends AbstractController
{

    public function index(PersistanceManagerRegistry $doctrine): Response
    {
        $em = $doctrine->getManager();
        $eventos = $em->getRepository('App\Entity\Eventos')->findAll();

        return $this->render('dashboard/index.html.twig', [
            'eventos' => $eventos,
        ]);
    }

    public function crearVenta(PersistanceManagerRegistry $doctrine, $mes, $year): Response
    {
        $em = $doctrine->getManager();
        $ventas = $em->getRepository('App\Entity\Ventas')->findBy(['mes' => intval($mes), 'year' => intval($year)]);

        if (empty($ventas)) {
            $venta = new Ventas();
        } else {
            $venta = $ventas[0];
        }
        $fechas = $this->getStartAndEndOfMonth($mes, $year);
        $pedidos = $em->getRepository('App\Entity\Pedidos')->totalPedidosMes($fechas[0], $fechas[1]);
        $total = 0.0;
        foreach ($pedidos as $pedido) {
            $total = $total + $pedido->getTotal();
        }
        $cancelados = $em->getRepository('App\Entity\Pedidos')->findBy(['estado' => 'cancelado']);

        $venta->setMes(intval($mes));
        $venta->setYear(intval($year));
        $venta->setTotal($total);
        $venta->setCompletados(count($pedidos));
        $venta->setCancelados(count($cancelados));

        $em->persist($venta);
        $em->flush();
        if (empty($ventas)) {
            $this->addFlash('success', 'Se ha creado el historial del mes correctamente');
        } else {
            $this->addFlash('success', 'Se ha actualizado el historial del mes correctamente');
        }
        return $this->redirect('/admin/dashboard');
    }

    public function datosDashboard(PersistanceManagerRegistry $doctrine, PedidosRepository $pedidosRepository): Response
    {
        $em = $doctrine->getManager();
        $actualMes = date('n');
        $actualAno = date('Y');
        $sum = $sumA = 0.0;
        if ( $actualMes == 1 ) {
            $mes = 12;
            $ano = $actualAno - 1;
        } else {
            $mes = $actualMes - 1;
            $ano = $actualAno;
        }

        $fechas = $this->getStartAndEndOfMonth($mes, $ano);
        $pedidosAnteriores = $em->getRepository('App\Entity\Pedidos')->totalPedidosMes($fechas[0], $fechas[1]);
        foreach ($pedidosAnteriores as $ped) {
            $sum = $sum + $ped->getTotal();
        }

        $fechasA = $this->getStartAndEndOfMonth($actualMes, $actualAno);
        $pedidosActuales = $em->getRepository('App\Entity\Pedidos')->totalPedidosMes($fechasA[0], $fechasA[1]);
        foreach ($pedidosActuales as $ped) {
            $sumA = $sumA + $ped->getTotal();
        }

        $pedidosActivos = $em->getRepository('App\Entity\Pedidos')->pedidosActivos();
        $totalPedidosActivo = count($pedidosActivos);

        $clientes = $em->getRepository('App\Entity\Clientes')->findBy(['activo' => true]);
        $clientesActivo = count($clientes);

        $fabricantes = $em->getRepository('App\Entity\Fabricantes')->findBy(['activo' => true]);
        $fabricantesActivo = count($fabricantes);

        $catalogos = $em->getRepository('App\Entity\Catalogos')->findBy(['activo' => true]);
        $catalogosActivo = count($catalogos);

        return $this->render('dashboard/datos_dashboard.html.twig', [
            'precioTotalAnterior' => $sum,
            'precioTotalActual' => $sumA,
            'totalPedidosActivo' => $totalPedidosActivo,
            'clientesActivo' => $clientesActivo,
            'fabricantesActivo' => $fabricantesActivo,
            'catalogosActivo' => $catalogosActivo,
            'mes' => $mes,
            'year' => $ano,
            'actualMes' => $actualMes,
            'actualYear' => $actualAno,
        ]);
    }

    private function getStartAndEndOfMonth($mes, $ano): array
    {
        $datString = $mes . '/01/' . $ano;
        if (in_array($mes, [1, 3, 5, 7, 8, 10, 12])) {
            $datString2 = $mes . '/31/' . $ano;
        } elseif ($mes == 2) {
            $datString2 = $mes . '/28/' . $ano;
            if ($ano % 4 == 0 && ($ano % 100 != 0 || $ano % 400 == 0)) {
                $datString2 = $mes . '/29/' . $ano;
            }
        } else {
            $datString2 = $mes . '/30/' . $ano;
        }

        $mesPI = new \Datetime($datString);
        $mesPF = new \Datetime($datString2);
        $mesPF->setTime(23, 59, 59);

        return [$mesPI, $mesPF];
    }
}