<?php

namespace App\Entity;

use App\Repository\ClientesRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * @ORM\Entity(repositoryClass=ClientesRepository::class)
 */
class Clientes
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $nombre;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $direccion;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $telefono;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $email;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $provincia;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $codigoPostal;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\User", mappedBy="cliente", cascade={"persist", "remove"})
     */
    private $usuarios;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\Pedidos", mappedBy="cliente", cascade={"persist", "remove"})
     */
    private $pedidos;

    /**
     * @ORM\Column(type="boolean")
     */
    private $activo;

    public function __construct()
    {
        $this->usuarios = new ArrayCollection();
        $this->pedidos = new ArrayCollection();
    }


    public function getId(): ?int
    {
        return $this->id;
    }

    public function getNombre(): ?string
    {
        return $this->nombre;
    }

    public function setNombre(string $nombre): self
    {
        $this->nombre = $nombre;

        return $this;
    }

    public function getDireccion(): ?string
    {
        return $this->direccion;
    }

    public function setDireccion(?string $direccion): self
    {
        $this->direccion = $direccion;

        return $this;
    }

    public function getTelefono(): ?string
    {
        return $this->telefono;
    }

    public function setTelefono(?string $telefono): self
    {
        $this->telefono = $telefono;

        return $this;
    }

    public function getEmail(): ?string
    {
        return $this->email;
    }

    public function setEmail(?string $email): self
    {
        $this->email = $email;

        return $this;
    }

    public function getProvincia(): ?string
    {
        return $this->provincia;
    }

    public function setProvincia(?string $provincia): self
    {
        $this->provincia = $provincia;

        return $this;
    }

    public function getCodigoPostal(): ?string
    {
        return $this->codigoPostal;
    }

    public function setCodigoPostal(?string $codigoPostal): self
    {
        $this->codigoPostal = $codigoPostal;

        return $this;
    }

    public function getUsuarios()
    {
        return $this->usuarios;
    }

    public function addUsuario(User $usuario): self
    {
        if (!$this->usuarios->contains($usuario)) {
            $this->usuarios[] = $usuario;
            $usuario->setCliente($this);
        }

        return $this;
    }

    public function removeUsuario(User $usuario): self
    {
        if ($this->usuarios->removeElement($usuario)) {
            if ($usuario->getCliente() === $this) {
                $usuario->setCliente(null);
            }
        }

        return $this;
    }

    public function getPedidos()
    {
        return $this->pedidos;
    }

    public function addPedido(Pedidos $pedido): self
    {
        if (!$this->pedidos->contains($pedido)) {
            $this->pedidos[] = $pedido;
            $pedido->setCliente($this);
        }

        return $this;
    }

    public function removePedido(Pedidos $pedido): self
    {
        if ($this->pedidos->removeElement($pedido)) {
            if ($pedido->getCliente() === $this) {
                $pedido->setCliente(null);
            }
        }

        return $this;
    }

    public function getActivo()
    {
        return $this->activo;
    }

    public function setActivo($activo): self
    {
        $this->activo = $activo;

        return $this;
    }

    public static function getProvinciasLista(): array
    {
        return [
            'Álava' => 'Álava',
            'Albacete' => 'Albacete',
            'Alicante' => 'Alicante',
            'Almería' => 'Almería',
            'Asturias' => 'Asturias',
            'Ávila' => 'Ávila',
            'Badajoz' => 'Badajoz',
            'Baleares' => 'Baleares',
            'Barcelona' => 'Barcelona',
            'Burgos' => 'Burgos',
            'Cáceres' => 'Cáceres',
            'Cádiz' => 'Cádiz',
            'Cantabria' => 'Cantabria',
            'Castellón' => 'Castellón',
            'Ciudad Real' => 'Ciudad Real',
            'Córdoba' => 'Córdoba',
            'Cuenca' => 'Cuenca',
            'Girona' => 'Girona',
            'Granada' => 'Granada',
            'Guadalajara' => 'Guadalajara',
            'Guipúzcoa' => 'Guipúzcoa',
            'Huelva' => 'Huelva',
            'Huesca' => 'Huesca',
            'Jaén' => 'Jaén',
            'La Rioja' => 'La Rioja',
            'Las Palmas' => 'Las Palmas',
            'León' => 'León',
            'Lleida' => 'Lleida',
            'Lugo' => 'Lugo',
            'Madrid' => 'Madrid',
            'Málaga' => 'Málaga',
            'Murcia' => 'Murcia',
            'Navarra' => 'Navarra',
            'Ourense' => 'Ourense',
            'Palencia' => 'Palencia',
            'Pontevedra' => 'Pontevedra',
            'Salamanca' => 'Salamanca',
            'Santa Cruz de Tenerife' => 'Santa Cruz de Tenerife',
            'Segovia' => 'Segovia',
            'Sevilla' => 'Sevilla',
            'Soria' => 'Soria',
            'Tarragona' => 'Tarragona',
            'Teruel' => 'Teruel',
            'Toledo' => 'Toledo',
            'Valencia' => 'Valencia',
            'Valladolid' => 'Valladolid',
            'Vizcaya' => 'Vizcaya',
            'Zamora' => 'Zamora',
            'Zaragoza' => 'Zaragoza'
        ];
    }
}
