<?php

namespace App\Entity;

use App\Repository\PedidosRepository;
use Doctrine\ORM\Mapping as ORM;
use Doctrine\Common\Collections\ArrayCollection;
use Gedmo\Timestampable\Traits\TimestampableEntity;

/**
 * @ORM\Entity(repositoryClass=PedidosRepository::class)
 */
class Pedidos
{
    use TimestampableEntity;

    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    private $id;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $identificador;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $direccion;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $codigoPostal;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $ciudad;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $condiciones;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $partes;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $banco;

    /**
     * @ORM\Column(type="string", length=255, nullable=true)
     */
    private $dni;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $cuentaBanco;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $observaciones;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Fabricantes", inversedBy="pedidos")
     * @ORM\JoinColumn(nullable=true)
     */
    private $fabricante;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Clientes", inversedBy="pedidos")
     * @ORM\JoinColumn(nullable=true)
     */
    private $cliente;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\PedidoLineas", mappedBy="pedido", cascade={"persist", "remove"})
     */
    private $lineas;

    /**
     * @ORM\Column(type="string", length=255)
     */
    private $estado;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\PedidoHistorial", mappedBy="pedido", cascade={"persist", "remove"})
     */
    private $historial;

    /**
     * @ORM\Column(type="float", nullable=true)
     */
    private $total;


    public function __construct()
    {
        $this->lineas = new ArrayCollection();
        $this->historial = new ArrayCollection();
    }

    public function getId(): ?int
    {
        return $this->id;
    }

    public function getIdentificador(): ?string
    {
        return $this->identificador;
    }

    public function setIdentificador(string $identificador): self
    {
        $this->identificador = $identificador;

        return $this;
    }

    public function getDireccion(): ?string
    {
        return $this->direccion;
    }

    public function setDireccion(string $direccion): self
    {
        $this->direccion = $direccion;

        return $this;
    }

    public function getCodigoPostal(): ?string
    {
        return $this->codigoPostal;
    }

    public function setCodigoPostal(string $codigoPostal): self
    {
        $this->codigoPostal = $codigoPostal;

        return $this;
    }

    public function getCiudad(): ?string
    {
        return $this->ciudad;
    }

    public function setCiudad(string $ciudad): self
    {
        $this->ciudad = $ciudad;

        return $this;
    }

    public function getCondiciones(): ?string
    {
        return $this->condiciones;
    }

    public function setCondiciones(?string $condiciones): self
    {
        $this->condiciones = $condiciones;

        return $this;
    }

    public function getPartes(): ?string
    {
        return $this->partes;
    }

    public function setPartes(?string $partes): self
    {
        $this->partes = $partes;

        return $this;
    }

    public function getBanco(): ?string
    {
        return $this->banco;
    }

    public function setBanco(?string $banco): self
    {
        $this->banco = $banco;

        return $this;
    }

    public function getDni(): ?string
    {
        return $this->dni;
    }

    public function setDni(?string $dni): self
    {
        $this->dni = $dni;

        return $this;
    }

    public function getCuentaBanco(): ?string
    {
        return $this->cuentaBanco;
    }

    public function setCuentaBanco(string $cuentaBanco): self
    {
        $this->cuentaBanco = $cuentaBanco;

        return $this;
    }

    public function getObservaciones(): ?string
    {
        return $this->observaciones;
    }

    public function setObservaciones(string $observaciones): self
    {
        $this->observaciones = $observaciones;

        return $this;
    }

    public function getFabricante()
    {
        return $this->fabricante;
    }

    public function setFabricante($fabricante): self
    {
        $this->fabricante = $fabricante;

        return $this;
    }

    public function getCliente()
    {
        return $this->cliente;
    }

    public function setCliente($cliente): self
    {
        $this->cliente = $cliente;

        return $this;
    }

    public function getLineas()
    {
        return $this->lineas;
    }

    public function addLinea(PedidoLineas $linea): self
    {
        if (!$this->lineas->contains($linea)) {
            $this->lineas[] = $linea;
            $linea->setPedido($this);
        }

        return $this;
    }

    public function removeLinea(PedidoLineas $linea): self
    {
        if ($this->lineas->removeElement($linea)) {
            if ($linea->getPedido() === $this) {
                $linea->setPedido(null);
            }
        }

        return $this;
    }

    public function getEstado()
    {
        return $this->estado;
    }

    public function setEstado(?string $estado)
    {
        $this->estado = $estado;

        return $this;
    }

    public function getHistorial()
    {
        return $this->historial;
    }

    public function addHistorial(PedidoHistorial $historial): self
    {
        if (!$this->historial->contains($historial)) {
            $this->historial[] = $historial;
            $historial->setPedido($this);
        }

        return $this;
    }

    public function removeHistorial(PedidoHistorial $historial): self
    {
        if ($this->historial->removeElement($historial)) {
            if ($historial->getPedido() === $this) {
                $historial->setPedido(null);
            }
        }

        return $this;
    }

    public function getTotal(): ?float
    {
        return $this->total;
    }

    public function setTotal(?float $total): self
    {
        $this->total = $total;

        return $this;
    }
}
