<?php

namespace App\Entity;

use Doctrine\ORM\Mapping as ORM;
use Sonata\UserBundle\Entity\BaseUser;
use Doctrine\Common\Collections\ArrayCollection;

/**
 * @ORM\Entity()
 */
class User extends BaseUser
{
    /**
     * @ORM\Id
     * @ORM\GeneratedValue
     * @ORM\Column(type="integer")
     */
    protected $id;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Fabricantes", inversedBy="usuarios")
     * @ORM\JoinColumn(nullable=true)
     */
    private $fabricante;

    /**
     * @ORM\ManyToOne(targetEntity="App\Entity\Clientes", inversedBy="usuarios")
     * @ORM\JoinColumn(nullable=true)
     */
    private $cliente;

    /**
     * @ORM\OneToMany(targetEntity="App\Entity\PedidoHistorial", mappedBy="usuario", cascade={"persist", "remove"})
     */
    private $historialPedidos;


    public function __construct()
    {
        $this->historialPedidos = new ArrayCollection();
    }

    public function getFabricante()
    {
        return $this->fabricante;
    }

    public function setFabricante($fabricante): self
    {
        $this->fabricante = $fabricante;

        return $this;
    }

    public function getCliente()
    {
        return $this->cliente;
    }

    public function setCliente($cliente): self
    {
        $this->cliente = $cliente;

        return $this;
    }

    public function getHistorialPedidos()
    {
        return $this->historialPedidos;
    }

    public function addHistorialPedido(PedidoHistorial $historialPedido): self
    {
        if (!$this->historialPedidos->contains($historialPedido)) {
            $this->historialPedidos[] = $historialPedido;
            $historialPedido->setUsuario($this);
        }

        return $this;
    }

    public function removeHistorialPedido(PedidoHistorial $historialPedido): self
    {
        if ($this->historialPedidos->removeElement($historialPedido)) {
            // set the owning side to null (unless already changed)
            if ($historialPedido->getUsuario() === $this) {
                $historialPedido->setUsuario(null);
            }
        }

        return $this;
    }

}