<?php

namespace App\EventListener;

use Doctrine\ORM\Event\LifecycleEventArgs;
use Doctrine\ORM\Event\PostFlushEventArgs;
use App\Entity\Pedidos;
use App\Entity\PedidoHistorial;
use Symfony\Component\Security\Core\Authentication\Token\Storage\UsageTrackingTokenStorage;
use App\Services\Mailer;

class PedidoListener
{
    private $tokenStorage;
    private $mailer;
    public $historial = null;

    public function __construct(UsageTrackingTokenStorage $tokenStorage, Mailer $mailer)
    {
        $this->tokenStorage = $tokenStorage;
        $this->mailer = $mailer;
    }

    public function preUpdate(LifecycleEventArgs $args)
    {
        $em = $args->getEntityManager();
        $entity = $args->getEntity();

        if ($entity instanceof Pedidos) {

            $lineas = $entity->getLineas();
            $sum = 0.0;
            foreach ($lineas as $linea) {
                $sum = $sum + $linea->getPrecioTotal();
            }
            $entity->setTotal($sum);

            if ($args->hasChangedField('estado')) {
                
                $user = $this->tokenStorage->getToken()->getUser();
                $this->historial = new PedidoHistorial();
                $this->historial->setEstado($args->getNewValue('estado'));
                $this->historial->setPedido($entity);
                $this->historial->setUsuario($user);
                
                if ($args->getNewValue('estado') == 'enviado') {

                    $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'enviadoCliente']);
                    $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
                    $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'enviadoFabricante']);
                    $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);

                } elseif ($args->getNewValue('estado') == 'cancelado') {

                    $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'cancelado']);
                    $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
                    $this->mailer->sendEmail($entity->getFabricante()->getEmail(), $email, $entity);

                } elseif ($args->getNewValue('estado') == 'pendiente') {

                    $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'pendiente']);
                    $this->mailer->sendEmail($entity->getFabricante()->getEmail(), $email, $entity);

                } else {
                    $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => $args->getNewValue('estado')]);
                    $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
                }
                
            }
        }
    }

    public function prePersist(LifecycleEventArgs $args)
    {
        $em = $args->getEntityManager();
        $entity = $args->getEntity();

        if ($entity instanceof Pedidos) {

            $lineas = $entity->getLineas();
            $sum = 0.0;
            foreach ($lineas as $linea) {
                $sum = $sum + $linea->getPrecioTotal();
            }
            $entity->setTotal($sum);

            $user = $this->tokenStorage->getToken()->getUser();
            $this->historial = new PedidoHistorial();
            $this->historial->setEstado($entity->getEstado());
            $this->historial->setPedido($entity);
            $this->historial->setUsuario($user);

            if ($entity->getEstado('estado') == 'enviado') {

                $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'enviadoCliente']);
                $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
                $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'enviadoFabricante']);
                $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);

            } elseif ($entity->getEstado('estado') == 'cancelado') {

                $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'cancelado']);
                $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
                $this->mailer->sendEmail($entity->getFabricante()->getEmail(), $email, $entity);

            } elseif ($entity->getEstado('estado') == 'pendiente') {

                $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => 'pendiente']);
                $this->mailer->sendEmail($entity->getFabricante()->getEmail(), $email, $entity);

            } else {
                $email = $em->getRepository('App\Entity\Emails')->findOneBy(['tipo' => $entity->getEstado('estado')]);
                $this->mailer->sendEmail($entity->getCliente()->getEmail(), $email, $entity);
            }
        }
    }

    public function postFlush(PostFlushEventArgs $args)
    {
        if (!empty($this->historial)) {
            $em = $args->getEntityManager();
            $em->persist($this->historial);
            $this->historial = null;
            $em->flush();
        }
    }


}