<?php

namespace App\Services;

use Symfony\Component\Mailer\MailerInterface;
use Symfony\Component\Mime\Email;
use Psr\Log\LoggerInterface;
use Twig\Environment;

class Mailer
{
    private $mailer;

    public function __construct(MailerInterface $mailer, LoggerInterface $logger, Environment $twig)
    {
        $this->mailer = $mailer;
        $this->logger = $logger;
        $this->twig = $twig;
    }

    public function sendEmail($to, $entidad, $pedido)
    {
        $email = (new Email())
            ->from('eortegamorales00@gmail.com')
            ->to($to)
            ->subject($entidad->getAsunto())
            ->html($this->twig->render('email/variables_email.html.twig', [
                'entidad' => $entidad,
                'pedido' => $pedido,
            ]));
        
        try {
            $this->mailer->send($email);
            $this->logger->info('Email sent successfully.');
            return true;
        } catch (TransportExceptionInterface $e) {
            $this->logger->error('Failed to send email: ' . $e->getMessage());
            return false;
        }
    }

    public function sendRecordatorio($eventos, $dia)
    {
        $email = (new Email())
            ->from('eortegamorales00@gmail.com')
            ->to('ortegamoralesedu@gmail.com')
            ->subject('Recordatorio Evento')
            ->html($this->twig->render('email/recordatorio_evento.html.twig', [
                'eventos' => $eventos,
                'dia' => $dia,
            ]));
        
        try {
            $this->mailer->send($email);
            $this->logger->info('Email reminder sent successfully.');
            return true;
        } catch (TransportExceptionInterface $e) {
            $this->logger->error('Failed to send reminder: ' . $e->getMessage());
            return false;
        }
    }
}